#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include "synchronize.h"

sem_t *INIT(Mutex *mutex, char *name)
{
    mutex->name = name;
    mutex->mtx  = sem_open(name, O_CREAT, 0666, 1);

    if (mutex->mtx == SEM_FAILED)
    {
        printf("*** Mutex '%s' init error!\n", name);
        exit(-11);
    }

    return mutex->mtx;
}

sem_t *USE(Mutex *mutex, char *name)
{
    mutex->name = name;
    mutex->mtx  = sem_open(name, 0);

    if (mutex->mtx == SEM_FAILED)
    {
        printf("*** Mutex '%s' use error!\n", name);
        exit(-12);
    }

    return mutex->mtx;
}

void LOCK(Mutex *mutex)
{
    if (sem_wait(mutex->mtx) == -1)
    {
        printf("*** Mutex '%s' lock error!\n", mutex->name);
        exit(-13);
    }
}

void UNLOCK(Mutex *mutex)
{
    if (sem_post(mutex->mtx) == -1)
    {
        printf("*** Mutex '%s' unlock error!\n", mutex->name);
        exit(-14);
    }
}

void REMOVE(char *name)
{
    if ((sem_unlink(name) == -1) && (errno != ENOENT))
    {
        printf("*** Semaphore '%s' unlink error!\n", name);
        exit(-21);
    }
}

sem_t *CREATE(Semaphore *semaphore, char *name, int value)
{
    semaphore->name = name;
    semaphore->sem  = sem_open(name, O_CREAT, 0666, value);

    if (semaphore->sem == SEM_FAILED)
    {
        printf("*** Semaphore '%s' create error!\n", name);
        exit(-22);
    }

    return semaphore->sem;
}

sem_t *OPEN(Semaphore *semaphore, char *name)
{
    semaphore->name = name;
    semaphore->sem  = sem_open(semaphore->name, 0);

    if (semaphore->sem == SEM_FAILED)
    {
        printf("*** Semaphore '%s' open error!\n", semaphore->name);
        exit(-23);
    }

    return semaphore->sem;
}

void WAIT(Semaphore *semaphore)
{
    if (sem_wait(semaphore->sem) == -1)
    {
        printf("*** Semaphore '%s' wait error!\n", semaphore->name);
        exit(-24);
    }
}

void SIGNAL(Semaphore *semaphore)
{
    if (sem_post(semaphore->sem) == -1)
    {
        printf("*** Semaphore '%s' signal error!\n", semaphore->name);
        exit(-25);
    }
}
